#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

#define BUF_SIZE 500

int
main(int argc, char *argv[])
{
    struct addrinfo hints;
    struct addrinfo *result, *rp;
    int s;

    if (argc < 3) {
        fprintf(stderr, "Usage: %s host port msg...\n", argv[0]);
        exit(EXIT_FAILURE);
    }

    /* Obtain address(es) matching host/port */

    memset(&hints, 0, sizeof(struct addrinfo));
    hints.ai_family = 0;    /* Allow IPv4 or IPv6 */
    hints.ai_socktype = SOCK_STREAM; /* Datagram socket */
    /* hints.ai_flags = 0; */
    hints.ai_flags = AI_CANONNAME;    /* For wildcard IP address */
    hints.ai_protocol = 0;          /* Any protocol */
    hints.ai_canonname = NULL;
    hints.ai_addr = NULL;
    hints.ai_next = NULL;

    s = getaddrinfo(argv[1], argv[2], &hints, &result);
    if (s != 0) {
        fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(s));
        exit(EXIT_FAILURE);
    }

    /* getaddrinfo() returns a list of address structures.
       Try each address until we successfully connect(2).
       If socket(2) (or connect(2)) fails, we (close the socket
       and) try the next address. */

    for (rp = result; rp != NULL; rp = rp->ai_next) {
        printf("ai_flags:    %d\n", rp->ai_flags);
        printf("ai_family:   %d\n", rp->ai_family);
        printf("ai_socktype: %d\n", rp->ai_socktype);
        printf("ai_protocol: %d\n", rp->ai_protocol);
        printf("ai_addrlen:  %d\n", rp->ai_addrlen);
        printf("ai_addr.type: %d\n", rp->ai_addr->sa_family); /* PF_INET */
        if (rp->ai_addr->sa_family == AF_INET) {
            printf("sin_port: %d\n" , ntohs(((struct sockaddr_in *)rp->ai_addr)->sin_port));
            printf("sin_addr: %s\n", inet_ntoa(((struct sockaddr_in *)rp->ai_addr)->sin_addr));
        }
        printf("ai_canonname:[%s]\n", rp->ai_canonname);
    }

    freeaddrinfo(result);           /* No longer needed */

    exit(EXIT_SUCCESS);
}
